/*
   Copyright (c) 2014-2020 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"

// Pil includes
#include "pil.h"

#include "main.h"
#include "sci.h"
#include "dispatcher.h"
#include "power.h"

#include "hal.h"

extern volatile unsigned int IFR;
extern volatile unsigned int IER;

// function prototypes
extern void DeviceInit(Uint16 pllDiv);
extern void MemCopy(Uint16 *SourceAddr, Uint16 *SourceEndAddr, Uint16 *DestAddr);
extern void InitFlash();

// scope for external mode
#ifdef NUM_EXT_MODE_SIGNALS
#define SCOPE_MAX_TRACE_WIDTH_IN_WORDS (NUM_EXT_MODE_SIGNALS *2)
#define SCOPE_BUF_SIZE (EXT_MODE_BUFFER_SIZE + SCOPE_MAX_TRACE_WIDTH_IN_WORDS)
#endif

#if (SCOPE_MAX_TRACE_WIDTH_IN_WORDS > (SCOPE_BUF_SIZE/2))
#error Excessive number of scope (external mode) signals.
#endif

#ifdef SCOPE_BUF_SIZE
#pragma DATA_SECTION(ScopeBuffer, "scope")
uint16_t ScopeBuffer[SCOPE_BUF_SIZE];
extern void PIL_setAndConfigScopeBuffer(PIL_Handle_t aPilHandle, uint16_t* aBufPtr, uint16_t aBufSize, uint16_t aMaxTraceWidthInWords);
#endif

// linker addresses, needed to copy code from flash to ram
extern Uint16 RamfuncsLoadStart, RamfuncsLoadEnd, RamfuncsRunStart;

PIL_Obj_t PilObj;
PIL_Handle_t PilHandle;

#ifdef SCI_CONFIGURED
SCI_Obj_t SciObj;
SCI_Handle_t SciHandle;
#endif

DISPR_TaskObj_t Tasks[NUM_TASKS];

static void SciPoll(PIL_Handle_t aHandle)
{
#ifdef SCI_CONFIGURED
    if(SCI_breakOccurred(SciHandle)){
        SCI_reset(SciHandle);
    }

    while(SCI_rxReady(SciHandle))
    {
        // assuming that there will be a "break" when FIFO is empty
        PIL_SERIAL_IN(aHandle, (int16)SCI_getChar(SciHandle));
    }

    int16_t ch;
    if(!SCI_txIsBusy(SciHandle)){
        if(PIL_SERIAL_OUT(aHandle, &ch))
        {
            SCI_putChar(SciHandle, ch);
        }
    }
#endif
}

void TaskB()
{
#ifdef CONTROL_BACKGROUND
    CONTROL_BACKGROUND;
#endif
}

void main(void)
{
	// low level hardware configuration
	DeviceInit(PLL_DIV);
#ifdef _FLASH
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);
	InitFlash();
#endif
	// disable all interrupts
	DINT;
	IER = 0x0000;
	IFR = 0x0000;

	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;      // Stop all the TB clocks
	EDIS;

#ifdef SCI_CONFIGURED
    // configure serial line
    SciHandle = SCI_init(&SciObj, sizeof(SciObj));
    SCI_configure(SciHandle, SCI_PORT, LSPCLK_HZ);
    (void)SCI_setupPort(SciHandle, SCI_BAUD_RATE);
#endif

	// initialize PIL framework
    PilHandle = PIL_init(&PilObj, sizeof(PilObj));
    PIL_setGuid(PilHandle, PIL_GUID_PTR);
#ifdef EXTERNAL_MODE_CHECKSUM_VAR
    PIL_setChecksum(PilHandle, EXTERNAL_MODE_CHECKSUM_VAR);
#endif
#ifdef PARALLEL_COM_PROTOCOL
    PIL_configureParallelCom(PilHandle, PARALLEL_COM_PROTOCOL, PARALLEL_COM_BUF_ADDR, PARALLEL_COM_BUF_LEN);
#else
    PIL_setSerialComCallback(PilHandle, (PIL_CommCallbackPtr_t)SciPoll);
#endif
#ifdef SCOPE_BUF_SIZE
    PIL_setAndConfigScopeBuffer(PilHandle, (uint16_t *)&ScopeBuffer, SCOPE_BUF_SIZE, SCOPE_MAX_TRACE_WIDTH_IN_WORDS);
#endif

#ifndef NO_PREP_TOOL
    PilInitOverrideProbes();
    PilInitCalibrations();
#endif

	// powerstage
	PWR_sinit();

	// HAL for controls
	HAL_sinit();
	HAL_configure();

    // dispatcher
    DISPR_sinit();
    DISPR_configure((uint32_t)(TASK1_PRD_IN_SYS_CLK_TICKS), PilHandle, &Tasks[0], sizeof(Tasks)/sizeof(DISPR_TaskObj_t));
    DISPR_registerIdleTask(&TaskB);
    DISPR_setPowerupDelay(10);

    // controls
    CONTROL_INIT;
    DISP_CONFIGURE_TRIGGER();

    // get ready for normal operation
    PIL_requestNormalMode(PilHandle);

    // enable interrupts
    EINT;   // global
    ERTM;   // real-time

    // go!
    DISPR_start(); // will not return

    PLX_ASSERT(0); // should never get here
}
