/*
   Copyright (c) 2020 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "cap.h"

CAP_Handle_t CAP_init(void *aMemory, const size_t aNumBytes)
{
    CAP_Handle_t handle;

    if(aNumBytes < sizeof(CAP_Obj_t))
        return((CAP_Handle_t)NULL);

    // set handle
    handle = (CAP_Handle_t)aMemory;

    return handle;
}

void CAP_setDefaultParams(uint16_t aNumEvents, bool aSingleShotOperation, CAP_Params_t *aParams)
{
    PLX_ASSERT(aSingleShotOperation);

    aParams->reg.ECCTL1.all = 0; // init to reset value
    aParams->reg.ECCTL2.all = 0x000E; // init to reset value

    aParams->reg.ECCTL2.bit.STOP_WRAP = aNumEvents-1;

    aParams->reg.ECCTL2.bit.CONT_ONESHT = 1;   // one-shot
    aParams->reg.ECCTL2.bit.SYNCI_EN = 0;      // disable sync in
    aParams->reg.ECCTL2.bit.SYNCO_SEL = 0;     // pass through
 }

void CAP_configure(CAP_Handle_t aHandle, uint16_t aUnit, uint16_t aGpio, const CAP_Params_t *aParams)
{
    CAP_Obj_t *obj = (CAP_Obj_t *)aHandle;

    PLX_ASSERT((aUnit >= 1) && (aUnit <= 7));

    EALLOW;
    switch(aUnit){
    case 1:
        obj->ecap = &ECap1Regs;
        CpuSysRegs.PCLKCR3.bit.ECAP1 = 1;
        ECap1Regs.ECCTL0.bit.INPUTSEL = 9; // cross-bar 10
        InputXbarRegs.INPUT10SELECT = aGpio;
        break;

    case 2:
        obj->ecap = &ECap2Regs;
        CpuSysRegs.PCLKCR3.bit.ECAP2 = 1;
        ECap2Regs.ECCTL0.bit.INPUTSEL = 10;
        InputXbarRegs.INPUT11SELECT = aGpio;
        break;

    case 3:
        obj->ecap = &ECap3Regs;
        CpuSysRegs.PCLKCR3.bit.ECAP3 = 1;
        ECap3Regs.ECCTL0.bit.INPUTSEL = 11;
        InputXbarRegs.INPUT12SELECT = aGpio;
        break;

    case 4:
        obj->ecap = &ECap4Regs;
        CpuSysRegs.PCLKCR3.bit.ECAP4 = 1;
        ECap4Regs.ECCTL0.bit.INPUTSEL = 12;
        InputXbarRegs.INPUT13SELECT = aGpio;
        break;

    case 5:
        obj->ecap = &ECap5Regs;
        CpuSysRegs.PCLKCR3.bit.ECAP5 = 1;
        ECap5Regs.ECCTL0.bit.INPUTSEL = 13;
        InputXbarRegs.INPUT14SELECT = aGpio;
        break;

    case 6:
        obj->ecap = &ECap6Regs;
        CpuSysRegs.PCLKCR3.bit.ECAP6 = 1;
        ECap6Regs.ECCTL0.bit.INPUTSEL = 14;
        InputXbarRegs.INPUT15SELECT = aGpio;
        break;

    case 7:
        obj->ecap = &ECap7Regs;
        CpuSysRegs.PCLKCR3.bit.ECAP7 = 1;
        ECap7Regs.ECCTL0.bit.INPUTSEL = 15;
        InputXbarRegs.INPUT16SELECT = aGpio;
        break;
    }

    obj->ecap->ECCTL1.bit.CAPLDEN = 0;  // disable CAP1-CAP4 register loads
    obj->ecap->ECCTL2.bit.TSCTRSTOP = 0;  // stop counter

    obj->ecap->ECEINT.all = 0x0000; // disable interrupts
    obj->ecap->ECCLR.all = 0xFFFF; // clear flags

    obj->ecap->ECCTL1.all = aParams->reg.ECCTL1.all;
    obj->ecap->ECCTL2.all = aParams->reg.ECCTL2.all;

    obj->ecap->ECCTL2.bit.STOP_WRAP = aParams->reg.ECCTL2.bit.STOP_WRAP; // when to stop
    obj->ecap->ECCTL1.bit.CAPLDEN = 1; // enable unit

    obj->ecap->ECCTL2.bit.TSCTRSTOP = 1; // start counter
    obj->ecap->ECCTL2.bit.REARM = 1; // arm one-shot

    EDIS;

    obj->numEvents = aParams->reg.ECCTL2.bit.STOP_WRAP + 1;
    obj->eventMask = (1 << (obj->numEvents+1)) - 2;
}

bool CAP_getNewValues(CAP_Handle_t aHandle, uint16_t aNewPrescale, uint32_t *aValues, bool *aOverflowFlag)
{
    CAP_Obj_t *obj = (CAP_Obj_t *)aHandle;

    *aOverflowFlag = obj->ecap->ECFLG.bit.CTROVF;
    obj->ecap->ECCLR.bit.CTROVF = 1;

    if((obj->ecap->ECFLG.all & obj->eventMask) ==  obj->eventMask)
    {
        volatile uint32_t *cap = &obj->ecap->CAP1;
        int i;
        for(i=0; i<obj->numEvents; i++)
        {
            aValues[i] = cap[i];
        }
        obj->ecap->ECCLR.all = obj->eventMask;
        EALLOW;
        obj->ecap->ECCTL1.bit.PRESCALE = (aNewPrescale >> 1) & 0x1F;
        obj->ecap->ECCTL2.bit.REARM = 1; // effective in single-shot and continuous mode
        EDIS;
        return true;
    }
    return false;
}
