/*
   Copyright (c) 2014-2020 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "dispatcher.h"

#if (DISPR_MCU == 280049)
#include "f28004x_epwm_defines.h"
#endif
#if (DISPR_MCU == 28379)
#include "F2837xS_EPwm_defines.h"
#endif
#if (DISPR_MCU == 28335)
#define ET_CTR_PRDZERO  0x3
#endif

/*
 * Assumptions/simplifications:
 * 1) ADC configured to trigger ADCINT1
 * 2) ePWM configured to issue SOCA
 * 3) ePWM int on PIE group 3 (ok for epwm1-8)
 * 4) ePWM and CpuTimer0 clock = SYSCLK/2
 * 5) Use of CpuTimer0 is hardcoded
 */

// singleton (for now)
DISPR_Obj_t DisprObj;
DISPR_Handle_t DisprHandle;

// for diagnostics purposes
#ifndef PIL_PREP_TOOL
PIL_SYMBOL_DEF(DisprObj_ctrlPeriod, 0, 1.0, "");
PIL_SYMBOL_DEF(DisprObj_task2DispatchPeriod, 0, 1.0, "");
PIL_SYMBOL_DEF(DisprObj_thisCntdownTimestamp0, 0, 1, "");
PIL_SYMBOL_DEF(DisprObj_thisCntdownTimestamp1, 0, 1, "");
PIL_SYMBOL_DEF(DisprObj_thisCntdownTimestamp2, 0, 1, "");
PIL_SYMBOL_DEF(DisprObj_cntdownTimestamp1, 0, 1, "");
PIL_SYMBOL_DEF(DisprObj_cntdownTimestamp2, 0, 1, "");
PIL_SYMBOL_DEF(DisprObj_task1Margin, 0, 1, "[1]");
PIL_SYMBOL_DEF(DisprObj_task1WorstCaseMargin, 0, 1, "[1]");
PIL_SYMBOL_DEF(DisprObj_cpuLoadInPercent, 0, 1, "[1]");
PIL_SYMBOL_DEF(DisprObj_variablePeriodDetected, 0, 1, "[1]");
PIL_SYMBOL_DEF(DisprObj_pwmTBPRDWc, 0, 1, "[1]");

#endif

static interrupt void DISPR_dispatch();
static void DISPR_background();

void PIL_SCOPE_sample(PIL_Handle_t aPilHandle);

void DISPR_sinit()
{
    DisprHandle = (DISPR_Handle_t)&DisprObj;

    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    obj->powerupDelayIntTask1Ticks = 0;
    obj->timeBase =  DISPR_TimebaseIsUndefined;
    obj->numTasks = 0;
    obj->variablePeriodDetected = false;
    obj->pwmNominalTBPRD = 0;
    obj->pwmTBPRDWc = 0;
    obj->pwmTBPRD = 0;
    obj->idleTask = (DISPR_IdleTaskPtr_t)0;
}

void DISPR_configure(uint32_t aPeriodInSysClkTicks, PIL_Handle_t aPilHandle,
                     DISPR_TaskObj_t *aTskMemory, uint16_t aNumTasks)
{
    PLX_ASSERT(aNumTasks >= 1);
    PLX_ASSERT(aNumTasks <= 16); // number of tasks limited by size of tasksReadyFlags
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    obj->periodInSysClkTicks = aPeriodInSysClkTicks;
    obj->pilHandle = aPilHandle;
    obj->tskMemory = aTskMemory;
    obj->numTasks = aNumTasks;

#if (DISPR_MCU == 28379)
    if(ClkCfgRegs.PERCLKDIVSEL.bit.EPWMCLKDIV == 0)
    {
        // SYSCLK <= 100 MHz
        CpuTimer0Regs.TPR.all = 1; // set cpu timers to same clock as ePWM (SYSCLK/2)
        CpuTimer1Regs.TPR.all = 1;
        obj->dispClkPeriodInSysClkTicks = 2;
    }
    else
    {
        // SYSCLK > 100 MHz
        CpuTimer0Regs.TPR.all = 3; // set cpu timers to same clock as ePWM (SYSCLK/4)
        CpuTimer1Regs.TPR.all = 3;
        obj->dispClkPeriodInSysClkTicks = 4;
    }
#else
    obj->dispClkPeriodInSysClkTicks = 2; // 1/2 system clock
    CpuTimer0Regs.TPR.all = 1; // set cpu timer to same clock as ePWM (SYSCLK/2)
    CpuTimer1Regs.TPR.all = 1;
#endif
}

void DISPR_registerIdleTask(DISPR_IdleTaskPtr_t aTsk)
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    obj->idleTask = aTsk;
}

void DISPR_registerTask(uint16_t aTaskId, DISPR_TaskPtr_t aTsk, uint32_t aPeriodInSysClkTicks, void * const aParameters){
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    PLX_ASSERT(aTaskId < obj->numTasks);
    if(aTaskId == 0){
        // task 0 always called at dispatcher rate
        PLX_ASSERT(aPeriodInSysClkTicks == obj->periodInSysClkTicks);
    }
    obj->tskMemory[aTaskId].periodInSysClkTicks = aPeriodInSysClkTicks;
    obj->tskMemory[aTaskId].tsk = aTsk;
    obj->tskMemory[aTaskId].mask = (1 << aTaskId);
    obj->tskMemory[aTaskId].params = aParameters;
    obj->tskMemory[aTaskId].periodInDisprTicks = (uint16_t)(aPeriodInSysClkTicks/obj->periodInSysClkTicks);
    // only exact multiples allowed
    PLX_ASSERT(((uint32_t)obj->tskMemory[aTaskId].periodInDisprTicks*obj->periodInSysClkTicks) == aPeriodInSysClkTicks);
}

extern void DISPR_setupTimer(uint16_t aUnit, uint32_t aPeriodInSysClkTicks)
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    PLX_ASSERT(aUnit == 0);

    uint32_t prd =  aPeriodInSysClkTicks/obj->dispClkPeriodInSysClkTicks;

    CpuTimer0Regs.PRD.all = prd-1;
    CpuTimer0Regs.TCR.bit.TRB = 1; // reload period
    CpuTimer0Regs.TCR.bit.TIE = 1; // enable trigger to SOC
}

static void DISPR_configurePwm(PWM_Unit_t aPwmChannel)
{
    PLX_ASSERT((int)aPwmChannel >= (int)PWM_EPWM1);
#if (DISPR_MCU == 28379)
    PLX_ASSERT((int)aPwmChannel <= (int)PWM_EPWM12);
#endif
#if (DISPR_MCU == 280049)
    PLX_ASSERT((int)aPwmChannel <= (int)PWM_EPWM8);
#endif
#if (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
    PLX_ASSERT((int)aPwmChannel <= (int)PWM_EPWM6);
#endif
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    PWM_getRegisterBase(aPwmChannel, &obj->ePwm);
#if (DISPR_MCU == 28379)
    if(aPwmChannel <= 8)
    {
        obj->ePwmIntVector = (PINT *)((uint32_t)(&PieVectTable.EPWM1_INT) + ((uint32_t)aPwmChannel-1)*sizeof(PINT *));
    }
    else
    {
        obj->ePwmIntVector = (PINT *)((uint32_t)(&PieVectTable.EPWM9_INT) + ((uint32_t)aPwmChannel-9)*sizeof(PINT *));
    }
#else
    obj->ePwmIntVector = (PINT *)((uint32_t)(&PieVectTable.EPWM1_INT) + ((uint32_t)aPwmChannel-1)*sizeof(PINT *));
#endif
    obj->ePwmIerBitMask = (1 << (aPwmChannel-1));
}

#if (DISPR_MCU == 280049)
static void DISPR_configureAdc(AIN_Unit_t aAdcUnit)
{
    PLX_ASSERT((int)aAdcUnit >= (int)AIN_ADC_A);
    PLX_ASSERT((int)aAdcUnit <= (int)AIN_ADC_C);

    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    AIN_getRegisterBase(aAdcUnit, &obj->adc);

    obj->adcIntVector = (PINT *)((uint32_t)(&PieVectTable.ADCA1_INT) + ((uint32_t)aAdcUnit)*sizeof(PINT *));
    obj->adcPie1BitMask = (1 << aAdcUnit);
}
#endif

#if (DISPR_MCU == 28379)
static void DISPR_configureAdc(AIN_Unit_t aAdcUnit)
{
    PLX_ASSERT((int)aAdcUnit >= (int)AIN_ADC_A);
    PLX_ASSERT((int)aAdcUnit <= (int)AIN_ADC_D);

    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    AIN_getRegisterBase(aAdcUnit, &obj->adc);

    if(aAdcUnit <= 2)
    {
        // AIN_ADC_A - AIN_ADC_B
        obj->adcIntVector = (PINT *)((uint32_t)(&PieVectTable.ADCA1_INT) + ((uint32_t)aAdcUnit)*sizeof(PINT *));
        obj->adcPie1BitMask = (1 << aAdcUnit);
    }
    else
    {
        // AIN_ADC_D is shifted
        obj->adcIntVector =  (PINT *)&PieVectTable.ADCD1_INT;
        obj->adcPie1BitMask = (1 << 5);
    }
}
#endif

void DISPR_setPowerupDelay(uint16_t aTask1Ticks)
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    obj->powerupDelayIntTask1Ticks = aTask1Ticks;
}

void DISPR_setTriggerByAdcViaTimer(AIN_Unit_t aAdcUnit, uint16_t aEocChannel)
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    DISPR_stop();

#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
    DISPR_configureAdc(aAdcUnit);
#elif (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
    AIN_getRegisterBase(aAdcUnit, &obj->adc);
#else
#error Unsupported DISPR_MCU
#endif
    obj->timeBase = DISPR_TimebaseIsAdc;
    obj->baseClock = DISPR_BaseClockIsTimer;

    EALLOW;
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
    obj->adc->ADCINTSEL1N2.bit.INT1CONT = 0; // disable ADCINT1 Continuous mode
    obj->adc->ADCINTSEL1N2.bit.INT1SEL = aEocChannel; // setup EOC to trigger ADCINT1
    obj->adc->ADCINTSEL1N2.bit.INT1E = 1; // enable ADCINT1
    obj->adc->ADCCTL1.bit.INTPULSEPOS = 1; // ADCINT1 trips after AdcResults latch
    *obj->adcIntVector = &DISPR_dispatch;
#elif (DISPR_MCU == 28069)
    obj->adc->INTSEL1N2.bit.INT1CONT = 0; // disable ADCINT1 Continuous mode
    obj->adc->INTSEL1N2.bit.INT1SEL = aEocChannel; // setup EOC to trigger ADCINT1
    obj->adc->INTSEL1N2.bit.INT1E = 1; // enable ADCINT1
    obj->adc->ADCCTL1.bit.INTPULSEPOS = 1; // ADCINT1 trips after AdcResults latch
    PieVectTable.ADCINT1 = &DISPR_dispatch;
#elif (DISPR_MCU == 28335)
    PieVectTable.ADCINT = &DISPR_dispatch;
#else
#error Unsupported DISPR_MCU
#endif
    EDIS;
}

void DISPR_setTriggerByAdcViaPwm(AIN_Unit_t aAdcUnit, uint16_t aEocChannel, PWM_Unit_t aPwmChannel)
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    DISPR_stop();

    DISPR_configurePwm(aPwmChannel);

#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
    DISPR_configureAdc(aAdcUnit);
#elif (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
    AIN_getRegisterBase(aAdcUnit, &obj->adc);
#else
#error Unsupported DISPR_MCU
#endif

    obj->timeBase = DISPR_TimebaseIsAdc;
    obj->baseClock = DISPR_BaseClockIsPwm;

    // work here: should also work without PWM, should make sure only one SOCAEN is set
    obj->ePwm->ETSEL.bit.SOCAEN = 1; // enable SOC on A group

    EALLOW;
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
    obj->adc->ADCINTSEL1N2.bit.INT1CONT = 0; // disable ADCINT1 Continuous mode
    obj->adc->ADCINTSEL1N2.bit.INT1SEL = aEocChannel; // setup EOC to trigger ADCINT1
    obj->adc->ADCINTSEL1N2.bit.INT1E = 1; // enable ADCINT1
    obj->adc->ADCCTL1.bit.INTPULSEPOS = 1; // ADCINT1 trips after AdcResults latch
    *obj->adcIntVector = &DISPR_dispatch;
#elif (DISPR_MCU == 28069)
    obj->adc->INTSEL1N2.bit.INT1CONT = 0; // disable ADCINT1 Continuous mode
    obj->adc->INTSEL1N2.bit.INT1SEL = aEocChannel; // setup EOC to trigger ADCINT1
    obj->adc->INTSEL1N2.bit.INT1E = 1; // enable ADCINT1
    obj->adc->ADCCTL1.bit.INTPULSEPOS = 1; // ADCINT1 trips after AdcResults latch
    PieVectTable.ADCINT1 = &DISPR_dispatch;
#elif (DISPR_MCU == 28335)
    obj->adc->ADCTRL2.bit.INT_ENA_SEQ1 = 1;  // Enable SEQ1 interrupt (every EOS)
    PieVectTable.ADCINT = &DISPR_dispatch;
#else
#error Unsupported DISPR_MCU
#endif
    EDIS;
}

void DISPR_setTriggerByTimer()
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    DISPR_stop();

    obj->timeBase = DISPR_TimebaseIsTimer;
    obj->baseClock = DISPR_BaseClockIsTimer;

    CpuTimer0Regs.TCR.bit.TIE = 1;
    PieCtrlRegs.PIEIER1.bit.INTx7 = 1;

    EALLOW;
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
    PieVectTable.TIMER0_INT = &DISPR_dispatch;
#elif (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
    PieVectTable.TINT0 = &DISPR_dispatch;
#else
#error Unsupported DISPR_MCU
#endif
    EDIS;
}

void DISPR_setTriggerByPwm(PWM_Unit_t aPwmChannel, uint16_t aEventSel)
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    DISPR_stop();

    DISPR_configurePwm(aPwmChannel);

    obj->timeBase = DISPR_TimebaseIsPwm;
    obj->baseClock = DISPR_BaseClockIsPwm;

    obj->ePwm->ETSEL.bit.INTSEL = aEventSel;
    obj->ePwm->ETSEL.bit.INTEN = 1;  // enable INT

    PieCtrlRegs.PIEIER3.all |= obj->ePwmIerBitMask;

    EALLOW;
    *obj->ePwmIntVector = &DISPR_dispatch;
    EDIS;
}

void DISPR_reset()
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;
    uint16_t i;
    for(i=0; i<obj->numTasks; i++){
        obj->tskMemory[i].timer = 0;
    }
    obj->tasksReadyFlags = 0;
}

void DISPR_start()
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    PLX_ASSERT(obj->numTasks > 0);

    // stop CPU timers
    CpuTimer0Regs.TCR.bit.TSS = 1;
    CpuTimer0Regs.TPRH.all = 0;
    CpuTimer1Regs.TCR.bit.TSS = 1;
    CpuTimer1Regs.TPRH.all = 0;

    if(obj->baseClock == DISPR_BaseClockIsTimer)
    {
        obj->ctrlPeriod =  obj->tskMemory[0].periodInSysClkTicks/obj->dispClkPeriodInSysClkTicks; // timer at ePWM clock
        PLX_ASSERT(obj->tskMemory[0].periodInSysClkTicks == obj->dispClkPeriodInSysClkTicks*obj->ctrlPeriod);
        PLX_ASSERT(CpuTimer0Regs.PRD.all == (obj->ctrlPeriod-1));

        // configure timer for timing diagnostics
        CpuTimer1Regs.PRD.all = obj->ctrlPeriod-1;
        CpuTimer1Regs.TCR.bit.TRB = 1; // reload timer (sets to period)
        CpuTimer0Regs.TCR.bit.TRB = 1; // reload timer (sets to period)
    }
    else if(obj->baseClock == DISPR_BaseClockIsPwm)
    {
        PLX_ASSERT(obj->ePwm->TBCTL.bit.CTRMODE != TB_COUNT_DOWN);

        obj->pwmNominalTBPRD = obj->ePwm->TBPRD;
        obj->pwmTBPRDWc = obj->ePwm->TBPRD;
        obj->pwmTBPRD = obj->ePwm->TBPRD;

        if(obj->ePwm->TBCTL.bit.PHSEN){
            // a phase synchronization will break our fixed frequency margin calculation
            obj->variablePeriodDetected = true;
        }

        bool firstTriggerOnPeriod = false;
        bool doubleEdgeTrigger = false;
        uint32_t intClkPeriodInSysClkTicks;
        if(obj->ePwm->TBCTL.bit.CTRMODE == TB_COUNT_UPDOWN){
            // triangle
            intClkPeriodInSysClkTicks = obj->dispClkPeriodInSysClkTicks*2L*(uint32_t)obj->ePwm->TBPRD;
            if(obj->timeBase == DISPR_TimebaseIsAdc){
                if(obj->ePwm->ETSEL.bit.SOCASEL == ET_CTR_PRDZERO){
                    doubleEdgeTrigger = true;
                } else {
                    firstTriggerOnPeriod = (obj->ePwm->ETSEL.bit.SOCASEL == ET_CTR_PRD);
                }
             } else {
                if(obj->ePwm->ETSEL.bit.INTSEL == ET_CTR_PRDZERO){
                    doubleEdgeTrigger = true;
                } else {
                    firstTriggerOnPeriod = (obj->ePwm->ETSEL.bit.INTSEL == ET_CTR_PRD);
                }
            }
        } else {
            // sawtooth
            intClkPeriodInSysClkTicks = obj->dispClkPeriodInSysClkTicks*(uint32_t)(obj->ePwm->TBPRD+1);
        }

        if(doubleEdgeTrigger){
            PLX_ASSERT(obj->ePwm->TBCTL.bit.CTRMODE == TB_COUNT_UPDOWN);
            intClkPeriodInSysClkTicks = intClkPeriodInSysClkTicks/2;
        }

        uint16_t prd = (uint16_t)(obj->tskMemory[0].periodInSysClkTicks/intClkPeriodInSysClkTicks);
        if(((uint32_t)prd*intClkPeriodInSysClkTicks) != obj->tskMemory[0].periodInSysClkTicks)
        {
            PLX_ASSERT(0); // only exact multiples allowed
        }
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
        if(prd > 15) // type D, up to 15
#elif (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
        if(prd > 3)
#else
#error Unsupported DISPR_MCU
#endif
        {
            PLX_ASSERT(0); // ratio not supported
        }

        if(obj->timeBase == DISPR_TimebaseIsAdc)
        {
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
            obj->ePwm->ETPS.bit.SOCPSSEL = 1;
            obj->ePwm->ETSOCPS.bit.SOCAPRD2 = prd;
#elif (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
            obj->ePwm->ETPS.bit.SOCAPRD = prd;
#else
#error Unsupported DISPR_MCU
#endif
        }
        else
        {
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
            obj->ePwm->ETPS.bit.INTPSSEL = 1;
            obj->ePwm->ETINTPS.bit.INTPRD2 = prd;
#elif (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
            obj->ePwm->ETPS.bit.INTPRD = prd;
#else
#error Unsupported DISPR_MCU
#endif
        }

        // configure timer for timing diagnostics
        if(obj->ePwm->TBCTL.bit.CTRMODE == TB_COUNT_UPDOWN){
            // triangle
            obj->ctrlPeriod =  (uint32_t)prd * (uint32_t)obj->ePwm->TBPRD;
            if(!doubleEdgeTrigger){
                obj->ctrlPeriod = obj->ctrlPeriod * 2;
            }
            if(firstTriggerOnPeriod){
                CpuTimer1Regs.PRD.all = obj->ctrlPeriod - 1 - (uint32_t)obj->ePwm->TBPRD; // initial offset
                CpuTimer1Regs.TCR.bit.TRB = 1; // reload timer (initial offset)
                CpuTimer1Regs.PRD.all = obj->ctrlPeriod-1;
            } else {
                CpuTimer1Regs.PRD.all = obj->ctrlPeriod-1;
                CpuTimer1Regs.TCR.bit.TRB = 1; // reload timer
            }
        } else {
            // sawtooth
            obj->ctrlPeriod =  (uint32_t)prd * (uint32_t)(obj->ePwm->TBPRD+1);
            CpuTimer1Regs.PRD.all = obj->ctrlPeriod-1;
            CpuTimer1Regs.TCR.bit.TRB = 1; // reload timer
        }
    }

    obj->interruptNesting = 0;
    obj->cntdownTimestamp1 = 0xFFFFFFFF;
    obj->cntdownTimestamp2 = 0xFFFFFFFF;
    DISPR_reset();

    obj->powerupCountdown = obj->powerupDelayIntTask1Ticks;

    if(obj->timeBase == DISPR_TimebaseIsAdc)
    {
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
        PieCtrlRegs.PIEIER1.all |= obj->adcPie1BitMask;
#elif (DISPR_MCU == 28069)
        PieCtrlRegs.PIEIER1.bit.INTx1 = 1;
#elif (DISPR_MCU == 28335)
        PieCtrlRegs.PIEIER1.bit.INTx6 = 1;
#else
#error Unsupported DISPR_MCU
#endif
        PieCtrlRegs.PIEACK.all = PIEACK_GROUP1; // Acknowledge interrupt to PIE (all ADCs in group 1)
        IER |= M_INT1;
    }
    else if (obj->timeBase == DISPR_TimebaseIsPwm)
    {
        PieCtrlRegs.PIEIER3.all |= obj->ePwmIerBitMask;
        PieCtrlRegs.PIEACK.all = PIEACK_GROUP3; // Acknowledge interrupt to PIE
        IER |= M_INT3;
    }
    else // timer
    {
        PieCtrlRegs.PIEIER1.bit.INTx7 = 1;
        PieCtrlRegs.PIEACK.all = PIEACK_GROUP1; // Acknowledge interrupt to PIE
        IER |= M_INT1;
    }

    EALLOW;
    CpuTimer0Regs.TCR.bit.TSS = 0; // start cpu timer 0 (in case it is used for dispatching)
    CpuTimer1Regs.TCR.bit.TSS = 0; // start cpu timer 1 for timing diagnostics
#if (DISPR_MCU == 28069) || (DISPR_MCU == 28335)
    SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
#else
    CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1; // start all the timers synced
#endif
    EDIS;

    // power-up delay
    while(obj->powerupCountdown > 0)
    {
        // wait to, e.g., allow gatedrive charge-pump to stabilize
        continue;
    }

    for(;;)
    {
        PIL_backgroundCall(obj->pilHandle);
        DISPR_background();
        if(obj->idleTask)
        {
            obj->idleTask();
        }
    }
}

void DISPR_stop()
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    switch(obj->timeBase){
        default:
        case DISPR_TimebaseIsUndefined:
            break;

        case DISPR_TimebaseIsTimer:
            PieCtrlRegs.PIEIER1.bit.INTx7 = 0;
            break;

        case DISPR_TimebaseIsPwm:
            PieCtrlRegs.PIEIER3.all &= ~obj->ePwmIerBitMask;
            break;

        case DISPR_TimebaseIsAdc:
#if (DISPR_MCU == 280049) || (DISPR_MCU == 28379)
            PieCtrlRegs.PIEIER1.all &= ~obj->adcPie1BitMask;
#elif (DISPR_MCU == 28069)
            PieCtrlRegs.PIEIER1.bit.INTx1 = 0;
#elif (DISPR_MCU == 28335)
            PieCtrlRegs.PIEIER1.bit.INTx6 = 0;
#else
#error Unsupported DISPR_MCU
#endif
            break;
    }
}

static interrupt void DISPR_dispatch(){
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    obj->thisCntdownTimestamp0 = CpuTimer1Regs.TIM.all;

    // check for variable frequency operation
    if(obj->baseClock == DISPR_BaseClockIsPwm)
    {
        if(obj->ePwm->TBPRD != obj->pwmNominalTBPRD)
        {
            if(!obj->variablePeriodDetected){
                // reset diagnostics
                obj->cntdownTimestamp1 = 0xFFFFFFFF;
                obj->cntdownTimestamp2 = 0xFFFFFFFF;
                CpuTimer1Regs.PRD.all = 0xFFFFFFFF;
                obj->variablePeriodDetected = true;
            }
        }
        obj->pwmTBPRD = obj->ePwm->TBPRD;
        if(obj->pwmTBPRDWc > obj->ePwm->TBPRD)
        {
            obj->pwmTBPRDWc = obj->ePwm->TBPRD;
        }
        if(obj->variablePeriodDetected){
             CpuTimer1Regs.TCR.bit.TRB = 1; // reload timer
             obj->thisCntdownTimestamp0 = 0xFFFFFFFF;
        }
    }

    if(obj->timeBase == DISPR_TimebaseIsAdc)
    {
#if (DISPR_MCU == 28335)
        obj->adc->ADCTRL2.bit.RST_SEQ1 = 1;     // reset SEQ1
        obj->adc->ADCST.bit.INT_SEQ1_CLR = 1;   // clear INT SEQ1 bit
#else
        obj->adc->ADCINTFLGCLR.bit.ADCINT1 = 1; // clear ADCINT1 flag reinitialize for next SOC
#endif
        PieCtrlRegs.PIEACK.all = PIEACK_GROUP1; // acknowledge interrupt to PIE (All ADCS in group 1)
        IER |= M_INT1;
    }

    else if (obj->timeBase == DISPR_TimebaseIsPwm)
    {
        obj->ePwm->ETCLR.bit.INT = 1;  // clear INT flag for this timer
        PieCtrlRegs.PIEACK.all = PIEACK_GROUP3; // acknowledge interrupt to PIE
        IER |= M_INT3;
    }
    else
    {
        // timer
        CpuTimer0Regs.TCR.bit.TIF = 1;  // clear interrupt flag (seems to be optional)
        PieCtrlRegs.PIEACK.all = PIEACK_GROUP1; // acknowledge interrupt to PIE
        IER |= M_INT1;
    }

     // we return immediately if power-up delay has not yet expired
    if(obj->powerupCountdown > 0)
    {
        obj->powerupCountdown--;
        return;
    }

    obj->interruptNesting++;
    PLX_ASSERT(obj->interruptNesting <= obj->numTasks);

#ifndef PARALLEL_COM_PROTOCOL
    PIL_BEGIN_INT_CALL(obj->pilHandle);
#else
    PIL_beginInterruptCall(obj->pilHandle);
#endif

    obj->thisCntdownTimestamp1 = CpuTimer1Regs.TIM.all;
    CpuTimer1Regs.TCR.bit.TIF = 1; // clear flag as we will be checking for it at next timestamp

    int i;
    // determine which tasks should be dispatched
    for(i=0; i<obj->numTasks; i++)
    {
        if(obj->tskMemory[i].timer == 0)
        {
            if(i==0)
            {
                // we give highest priority task special treatment (to minimize latency)
                obj->tskMemory[0].tsk(obj->tskMemory[0].params);
                // this is not really thread-safe, as we could be sampling a variable while lower
                // priority task is in the process of modifying its value
                PIL_SCOPE_sample(obj->pilHandle);
            }
            else
            {
                // schedule dispatching
                PLX_ASSERT(!(obj->tasksReadyFlags & obj->tskMemory[i].mask)); // check for overrun
                obj->tasksReadyFlags |= obj->tskMemory[i].mask;
            }
        }
        obj->tskMemory[i].timer++;
        if(obj->tskMemory[i].timer == obj->tskMemory[i].periodInDisprTicks)
        {
             obj->tskMemory[i].timer = 0;
        }
    }

    obj->thisCntdownTimestamp2 = CpuTimer1Regs.TIM.all;
    if(CpuTimer1Regs.TCR.bit.TIF == 1){
        obj->thisCntdownTimestamp2 = 0; // CPU timer has already bottomed out
    }

    // update worst-case timestamp values
    if(obj->cntdownTimestamp1 > obj->thisCntdownTimestamp1)
    {
        obj->cntdownTimestamp1 = obj->thisCntdownTimestamp1;
    }

    if(obj->cntdownTimestamp2 > obj->thisCntdownTimestamp2)
    {
        obj->cntdownTimestamp2 = obj->thisCntdownTimestamp2;
    }

    // run scheduled lower priority tasks
    i = 1; // start with second task
    uint16_t ready = (obj->tasksReadyFlags >> 1);
    while(ready != 0)
    {
        if(ready & 1)
        {
            EINT; // re-enable interrupt to allow nesting
            obj->tskMemory[i].tsk(obj->tskMemory[i].params);
            DINT;
            obj->tasksReadyFlags &= (~obj->tskMemory[i].mask);
        }
        ready >>= 1;
        i++;
    }

    DINT; // // TI expects interrupts to be disabled before entering I$$REST
    obj->interruptNesting--;
}

static void DISPR_background()
{
    DISPR_Obj_t *obj = (DISPR_Obj_t *)DisprHandle;

    // perform margin calculations
    DINT;
    bool variablePeriod = obj->variablePeriodDetected;
    uint16_t pwmTBPRD = obj->pwmTBPRD; // or maybe obj->pwmTBPRDWc?
    uint32_t ts1Wc = obj->cntdownTimestamp2;
    uint32_t ts1 = obj->thisCntdownTimestamp2;
    EINT;

    if(!variablePeriod){
        obj->task1Margin = (float)ts1/(float)obj->ctrlPeriod;
        obj->task1WorstCaseMargin = (float)ts1Wc/(float)obj->ctrlPeriod;
    } else {
        // determine control period for which the timestamps were captured
        uint32_t controlPeriod;
        if(obj->baseClock == DISPR_BaseClockIsPwm)
        {
             if(obj->ePwm->TBCTL.bit.CTRMODE == TB_COUNT_UPDOWN){
                 // triangle
                 if(obj->timeBase == DISPR_TimebaseIsAdc)
                 {
                     if(obj->ePwm->ETSEL.bit.SOCASEL == ET_CTR_PRDZERO)
                     {
                         controlPeriod = (uint32_t)pwmTBPRD;
                     }
                     else
                     {
                         controlPeriod = 2L*(uint32_t)pwmTBPRD;
                     }
                 }
                 else
                 {
                     if(obj->ePwm->ETSEL.bit.INTSEL == ET_CTR_PRDZERO)
                     {
                         controlPeriod = (uint32_t)pwmTBPRD;
                     }
                     else
                     {
                         controlPeriod = 2L*(uint32_t)pwmTBPRD;
                     }
                 }
             }
             else
             {
                 // sawtooth
                 controlPeriod = (uint32_t)(pwmTBPRD+1);
             }
        }
        // in the case variable frequency, the CpuTimer1 starts counting at 0xFFFFFFFF
        uint32_t tsOffset = 0xFFFFFFFF - controlPeriod;
        if(ts1 > tsOffset){
            obj->task1Margin = (float)(ts1-tsOffset)/(float)controlPeriod;
        } else {
            obj->task1Margin = 0;
        }
        if(ts1Wc > tsOffset){
            obj->task1WorstCaseMargin = (float)(ts1Wc-tsOffset)/(float)controlPeriod;
        } else {
            obj->task1WorstCaseMargin = 0;
        }
    }

    float cpuLoad = 100.0*(1.-obj->task1Margin);
    DINT;
    obj->cpuLoadInPercent = cpuLoad;
    EINT;
}
