local Coder = { }

local T = require(Target.Name)
local U = require('CoderUtils')
local C = require('TI28xCommon')

local Registry = {
    BackgroundCode = {},
    BackgroundTaskCount = 0;
    BackgroundTaskLabels = {},
    BackgroundCodeDeclarations = StringList:new(),
    GenericInstances = {},
    NumDigitalOutBlocks = 0,
    NumDigitalInBlocks = 0,
    GpioUsed = {},
    NumPwmBlocks = 0,
    PwmConfigs = {},
    PwmSyncConfigs = {},
    NumAdcBlocks = 0,
    AdcConfigs = {},
    AdcSocCounter = {},
    AdcSocConfigs = {},
    NumQepBlocks = 0,
    QepConfigs = {},
    NumSpiBlocks = 0,
    SpiConfigs = {},
    NumDacBlocks = 0,
    DacConfigs = {},
    PwmGpio = {},
    NumReadProbes = 0,
    ReadProbeConfigs = {},
    NumOverrideProbes = 0,
    OverrideProbeConfigs = {},
    NumCalibrations = 0,
    CalibrationConfigs = {},
    PowerstageProtectionConfig = nil,
    NumTimerBlocks = 0,
    TimerConfigs = {},
    NumCapBlocks = 0,
    CapConfigs = {},
    NumCanPorts = 0,
    CanPortConfigs = {},
    NumCanHandles = 0,
    CanHandles = {},
    CanMailboxCounter = {},
    NumExternalSyncBlocks = 0,
    ModelTrigger = nil
}

function Coder.GetFromArrayOrScalar(field, index, majordim)
    return U.GetFromArrayOrScalar(field, index, majordim)
end

function Coder.Round(num, numDecimalPlaces)
    return U.Round(num, numDecimalPlaces)
end

function Coder.IsValidCName(name)
    return U.IsValidCName(name)
end

function Coder.GetInstallDir()
   local installDir = Target.Variables.BUILD_ROOT

    if Target.Variables.genOnly == 1 then
        installDir = Target.Variables.installDir:gsub('%"+', ''):gsub('\\+','/') -- remove quotes, make all forward slashes
        local tempDir = Target.Variables.BUILD_ROOT:gsub('%"+', ''):gsub('\\+','/') 
        if installDir..Target.Variables.BASE_NAME.."_codegen" == tempDir then -- when installDir entry is blank, returns current path
            installDir = Target.Variables.BUILD_ROOT --  use default codegen dir if installDir is blank
        end
    end
    return installDir
end

function Coder.CopyTemplateToBuildDir(template, destname, dict)
    local installDir = Coder.GetInstallDir()
    if not U.FileExists(installDir.."/") then
        return "The directory '%s' does not exist." % {installDir}
    end
    U.CopyTemplateFile(template, installDir .. "/" .. destname, dict)
end

function Coder.GetTargetParameters()
	return T.GetTargetParameters()
end

function Coder.GetNewBackgroundTaskCode(label)
    local code = StringList:new()
    Registry.BackgroundCode[Registry.BackgroundTaskCount] = code
    Registry.BackgroundTaskLabels[Registry.BackgroundTaskCount] = label
    Registry.BackgroundTaskCount = Registry.BackgroundTaskCount + 1
    return code
end

function Coder.GetNewBackgroundTaskDeclarations(label)
    return Registry.BackgroundCodeDeclarations
end

function Coder.RegisterGenericInstance(id)
    if Registry.GenericInstances[id] == nil then
        Registry.GenericInstances[id] = 0;
    end
    local ret = Registry.GenericInstances[id]
    Registry.GenericInstances[id] = ret+1
    return ret
end

function Coder.RegisterDigitalOutBlock()
    local ret = Registry.NumDigitalOutBlocks
    Registry.NumDigitalOutBlocks = ret+1
    return ret
end

function Coder.RegisterDigitalInBlock()
    local ret = Registry.NumDigitalInBlocks
    Registry.NumDigitalInBlocks = ret+1
    return ret
end

function Coder.RegisterPwmBlock(pwm, params, req)
    -- all PWM blocks must be in the same task
    for _, p in pairs(Registry.PwmConfigs) do 
        if(params["task_name"] ~= p["task_name"]) then
            return nil, "All PWM blocks must be associated with the same task."
        end
    end
    local error = C.SetPwmHardwareParams(pwm, params, req, T)
    if(error ~= nil) then
        return nil, error
    end
    local ret = Registry.NumPwmBlocks
    params["instance"] = ret
    Registry.NumPwmBlocks = ret+1
    Registry.PwmConfigs[pwm] = params
    return ret
end

function Coder.GetNumPwmBlocks()
    return Registry.NumPwmBlocks
end

function Coder.RegisterExternalSyncBlock()
    local numExtSyncIn = T.GetTargetParameters()['num_extsync_in']
    Registry.NumExternalSyncBlocks = Registry.NumExternalSyncBlocks + 1
    if Registry.NumExternalSyncBlocks > numExtSyncIn then
        if numExtSyncIn == 1 then
          return nil, "There can be only one External Sync block."
        else
          return nil, "There can be only %d External Sync blocks." % {numExtSyncIn}
        end
    end
    return Registry.NumExternalSyncBlocks
end

function Coder.RegisterPwmSyncMux(pwm, type, unit)
    local error
    if type:sub(1, #"gpio") == "gpio" then
        error = T.CheckGpioIsValidPwmSync(unit)
    end 
    if error ~= nil then
        return error
    end
    Registry.PwmSyncConfigs[pwm] = {type = '', unit = 0}
    Registry.PwmSyncConfigs[pwm]["type"] = type
    Registry.PwmSyncConfigs[pwm]["unit"] = unit
end

function Coder.RegisterAdcBlock(adc)
    if Registry.AdcConfigs[adc] == nil then
      local ret = Registry.NumAdcBlocks
      Registry.AdcConfigs[adc] = {channel = ret}
      Registry.AdcSocCounter[adc] = 0
      Registry.NumAdcBlocks = ret+1
      return ret
    else
        local adcId = 'ADC %s' % {string.char(65+adc)}
        return nil, "There can be only one %s block." % {adcId}
    end
end

function Coder.RegisterAdcSoc(adc, params, req)
    local soc = Registry.AdcSocCounter[adc]
    if soc == nil then
        return nil, "Attempting to configure SOC for uninitialized ADC."
    end
    Registry.AdcSocCounter[adc] = soc+1

    local error = C.SetAdcSocHardwareParams(adc, soc, params, req, T)
    if(error ~= nil) then
        return nil, error
    end

    local configs = Registry.AdcSocConfigs[adc]
    if(configs == nil) then
        configs = {}
        Registry.AdcSocConfigs[adc] = configs
    end
    configs[soc] = params
    return soc
end

function Coder.SetAdcSocTrigger(adc, soc, trig)
    local configs = Registry.AdcSocConfigs[adc]
    if(configs == nil) then
        return "Attempting to configure SOC trigger for uninitialized ADC."
    end
    local params = configs[soc]
    if(params == nil) then
        return "Attempting to configure SOC trigger for uninitialized SOC."
    end
    params["trig"] = trig
end

function Coder.RegisterQepBlock(qep, params, req)
    local error = C.SetQepHardwareParams(qep, params, req, T)
    if(error ~= nil) then
        return nil, error
    end
    local ret = Registry.NumQepBlocks
    params["instance"] = ret
    Registry.NumQepBlocks = ret+1
    Registry.QepConfigs[qep] = params
    return ret
end

function Coder.RegisterSpiBlock(spi, params, req)
    local error = C.SetSpiHardwareParams(spi, params, req, T)
    if(error ~= nil) then
        return nil, error
    end
    local ret = Registry.NumSpiBlocks
    params["instance"] = ret
    Registry.NumSpiBlocks = ret+1
    Registry.SpiConfigs[spi] = params
    return ret
end

function Coder.RegisterDacBlock(dac, params, req)
    local error = C.SetDacParams(dac, params, req, T)
    if(error ~= nil) then
        return nil, error
    end
    local ret = Registry.NumDacBlocks
    params["instance"] = ret
    Registry.NumDacBlocks = ret+1
    Registry.DacConfigs[dac] = params

    return ret
end

function Coder.RegisterCapBlock(cap, params, req)
    local error = C.SetCapHardwareParams(cap, params, req, T)
    if(error ~= nil) then
        return nil, error
    end
    local ret = Registry.NumCapBlocks
    params["instance"] = ret
    Registry.NumCapBlocks = ret+1
    Registry.CapConfigs[cap] = params
    return ret
end

function Coder.RegisterReadProbe(name, params)
    Registry.NumReadProbes = Registry.NumReadProbes+1
    Registry.ReadProbeConfigs[name] = params
end

function Coder.RegisterOverrideProbe(name, params)
    Registry.NumOverrideProbes = Registry.NumOverrideProbes+1
    Registry.OverrideProbeConfigs[name] = params
end

function Coder.RegisterCalibration(name, params)
    Registry.NumCalibrations = Registry.NumCalibrations+1
    Registry.CalibrationConfigs[name] = params
end

function Coder.RegisterPowerstageProtection(params)
    if Registry.PowerstageProtectionConfig ~= nil then
        return "There can be only one powerstage protection block."
    end
    Registry.PowerstageProtectionConfig = params
end

function Coder.RegisterTimer(params)
    if Registry.NumTimerBlocks ~= 0 then
        return "There can be only one timer block."
    end
    local error = C.SetTimerParams(0, params, T)
    if(error ~= nil) then
        return error
    end
    Registry.NumTimerBlocks = 1
    Registry.TimerConfigs[0] = params
end

function Coder.RegisterModelTrigger(trig)
    Registry.ModelTrigger = eval(trig)["modtrig"]
end

function Coder.GetCanPortHandle(canInterface)
	if Registry.CanHandles[canInterface] == nil then		
		local error = T.CheckCanConfigIsValid(canInterface)
		if error ~= nil then
			return nil, error
		end
		Registry.CanHandles[canInterface] = Registry.NumCanHandles
		Registry.NumCanHandles = Registry.NumCanHandles + 1
	end
	return Registry.CanHandles[canInterface]
end

function Coder.RegisterCanPort(canInterface, params, req)
	if Registry.CanPortConfigs[canInterface] == nil then
      Registry.CanPortConfigs[canInterface] = params
      Registry.NumCanPorts = Registry.NumCanPorts+1
      return Coder.GetCanPortHandle(canInterface)
    else
        local canId = 'CAN %s' % {string.char(65+canInterface)}
        return nil, "There can be only one instance of a %s block." % {canId}
    end
end

function Coder.CanPortIsConfigured(canInterface)
	return (Registry.CanPortConfigs[canInterface] ~= nil)
end

function Coder.RegisterCanMailbox(canInterface)
	if Registry.CanMailboxCounter[canInterface] == nil then 
		Registry.CanMailboxCounter[canInterface] = 0
	end
	
	local mbox = Registry.CanMailboxCounter[canInterface]
	if mbox == 32 then
		return nil, "Only 32 CAN mailboxes available."
	end
	Registry.CanMailboxCounter[canInterface] = mbox + 1	
	return mbox
end

function Coder.getPwmUpdateCode()
    if Registry.PowerstageProtectionConfig ~= nil then
        if Registry.PowerstageProtectionConfig["sync_issued"] == nil then
            Registry.PowerstageProtectionConfig["sync_issued"] = true
            return 'PWR_syncEnableSwitching();'
        end
    end
    return nil
end

function Coder.Initialize()
    local Resources = ResourceList:new()

    T.Configure(Resources, Registry)
    
    local HeaderDeclarations = [=[
            typedef int_fast8_t int8_t;
            typedef uint_fast8_t uint8_t;
            ]=]
    HeaderDeclarations = HeaderDeclarations .. "extern void %s_background(void);\n" % {Target.Variables.BASE_NAME}

    return {
        HeaderDeclarations = HeaderDeclarations,
        Resources = Resources,
   }
end

function Coder.Finalize()
    local Include = StringList:new()
    local Declarations = StringList:new()
    local PreInitCode = StringList:new()
    local PostInitCode = StringList:new()
    local TerminateCode = StringList:new()
    local Require = ResourceList:new()
    local open = io.open
    
    local error
        
    Include:append('hal.h')
    Include:append('dispatcher.h')
       
    -- preliminary checks
    if (math.floor(1/Target.Variables.SAMPLE_TIME+0.5)-(1/Target.Variables.SAMPLE_TIME))>4e-9 then
        -- this is because the dispatcher uses uint32_t variables to represent frequencies. 4e-9 ~= eps(25e6)
        return "Fractional sample frequency (%f Hz) is not supported." % {1/Target.Variables.SAMPLE_TIME}
    end

    -- determine final destination for generated files
    local installDir = Coder.GetInstallDir()
    if not U.FileExists(installDir.."/") then
        return "The directory '%s' does not exist." % {installDir}
    end

    -- PIL probes and calibrations
    local probes = io.open(Target.Variables.BUILD_ROOT .. "/probes.tmp", "w")
    io.output(probes)
    io.write("#ifndef _PROBES_CG_H_\n")
    io.write("#define _PROBES_CG_H_\n")

    io.write('#define NO_PREP_TOOL\n')
    io.write(string.format("#define CODE_GUID %s\n", U.guid()))
    io.write(string.format('#define COMPILE_TIME_DATE_STR "%s"\n', os.date("%m/%d/%Y %I:%M %p")))
    io.write('#define USER_NAME "PLECS Coder"\n');

    if (Registry.NumReadProbes > 0) or (Registry.NumOverrideProbes > 0) or (Registry.NumCalibrations > 0) then
        Declarations:append("%s_t %s;\n" % {Target.Variables.BASE_NAME, Target.Variables.BASE_NAME})

        io.write("// PIL Probes\n")
        io.write("typedef struct {\n")

        -- override probes
        for k, v in pairs(Registry.OverrideProbeConfigs) do
            local type = v['type']
            if type == nil then
                type = 'float'
            end
            io.write(string.format("  %s %s;\n", type, k))
            io.write(string.format("  %s %s_probeV;\n", type, k))
            io.write(string.format("  %s %s_probeF;\n", "int16_t", k))
        end
        -- read probes
        for k, v in pairs(Registry.ReadProbeConfigs) do
            local type = v['type']
            if type == nil then
                type = 'float'
            end
            io.write(string.format("  %s %s;\n", type, k))
        end
        -- calibrations
        for k, v in pairs(Registry.CalibrationConfigs) do
            local type = v['type']
            if type == nil then
                type = 'float'
            end
            io.write(string.format("  %s %s;\n", type, k))
        end

        io.write(string.format("} %s_t;\n",Target.Variables.BASE_NAME))
        io.write(string.format("extern %s_t %s;\n",Target.Variables.BASE_NAME, Target.Variables.BASE_NAME))
        io.write("\n");

        -- override probes
        for k, v in pairs(Registry.OverrideProbeConfigs) do
            local type = v['type']
            if type == nil then
                type = 'float'
            end
            Declarations:append("PIL_SYMBOL_DEF(%s_%s, 0, 1.0, \"\");\n" % {Target.Variables.BASE_NAME, k})
            PreInitCode:append("INIT_OPROBE(%s.%s);\n" % {Target.Variables.BASE_NAME, k})
        end
        -- read probes
        for k, v in pairs(Registry.ReadProbeConfigs) do
            local type = v['type']
            if type == nil then
                type = 'float'
            end
            Declarations:append("PIL_SYMBOL_DEF(%s_%s, 0, 1.0, \"\");\n" % {Target.Variables.BASE_NAME, k})
        end
        -- calibrations
        for k, v in pairs(Registry.CalibrationConfigs) do
            local type = v['type']
            if type == nil then
                type = 'float'
            end
            Declarations:append("PIL_SYMBOL_CAL_DEF(%s_%s, 0, 1.0, \"\", %f, %f, %f);\n" % {Target.Variables.BASE_NAME, k, v['min'], v['max'], v['val']})
            PreInitCode:append("%s.%s = %f;\n" % {Target.Variables.BASE_NAME, k, v['val']})
        end

        Include:append('pil.h')
        Include:append("%s_%s" % {Target.Variables.BASE_NAME, 'probes_cg.h'})
    end

    if(Target.Variables.EXTERNAL_MODE ~= 0) then
        io.write('#define EXTERNAL_MODE 1\n')
        io.write("#define EXTERNAL_MODE_CHECKSUM_VAR %s_checksum\n" % {Target.Variables.BASE_NAME})
    else
        io.write('#undef EXTERNAL_MODE\n')
    end
    io.write("#endif // _PROBES_CG_H_\n")
    io.close(probes)
    U.CopyTemplateFile(Target.Variables.BUILD_ROOT .. "/probes.tmp", "%s/%s_probes_cg.h" % {installDir, Target.Variables.BASE_NAME})
    os.remove(Target.Variables.BUILD_ROOT .. "/probes.tmp")

    -- external mode helper symbols
    if(Target.Variables.EXTERNAL_MODE ~= 0) then
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_targetFloat_Size, sizeof(%s_FloatType));\n" % {Target.Variables.BASE_NAME})
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_targetPointer_Size, sizeof(%s_FloatType));\n" % {Target.Variables.BASE_NAME})
        -- note: the following assumes that the base-task sample time is the first value of the _sampleTime array
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_sampleTime_Ptr, (uint32_t)&%s_sampleTime);\n" % {Target.Variables.BASE_NAME})
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_checksum_Ptr, (uint32_t)&%s_checksum);\n" % {Target.Variables.BASE_NAME})
        Declarations:append("#if defined(%s_NumTunableParameters) && (%s_NumTunableParameters >0)\n" % {Target.Variables.BASE_NAME, Target.Variables.BASE_NAME})
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_P_Ptr, (uint32_t)&%s_P);\n" % {Target.Variables.BASE_NAME})
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_P_Size, (uint32_t)%s_NumTunableParameters);\n" % {Target.Variables.BASE_NAME})
        Declarations:append("#endif\n")
        Declarations:append("#if defined(%s_NumExtModeSignals) && (%s_NumExtModeSignals > 0)\n" % {Target.Variables.BASE_NAME, Target.Variables.BASE_NAME})
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_ExtModeSignals_Ptr, (uint32_t)&%s_ExtModeSignals[0]);\n" % {Target.Variables.BASE_NAME})
        Declarations:append("PIL_CONFIG_DEF(uint32_t, ExtMode_ExtModeSignals_Size, (uint32_t)%s_NumExtModeSignals);\n" % {Target.Variables.BASE_NAME})
        Declarations:append("#endif\n")
    end
    
    error = C.ConfigureTimers(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end

    error = C.ConfigureAdcSocTriggers(Registry, T)
    if error ~= nil then
        return error
    end

    error = C.ConfigureModelTrigger(Registry, T)
    if error ~= nil then
        return error
    end
    
    -- powerstage protection (has to be configured before PWMs
    if Registry.PowerstageProtectionConfig ~= nil then
        -- determine rate at which block is executed
        local ts = Registry.PowerstageProtectionConfig["sample_time"];
        if ts[1] == 0 then
            return "Invalid sample time for Powerstage Protection block."
        end
        local ps_rate = math.floor(1/ts[1]+0.5)

        Declarations:append("static DIO_Obj_t DoutDrvEnableObj;\n")
        Declarations:append("static GDRV_Obj_t GdrvObj;\n")
        Declarations:append("static GDRV_Handle_t GdrvHandle;\n")

        PreInitCode:append("{\n") 
        PreInitCode:append("  // gate-driver and protections\n") 
        PreInitCode:append("  GdrvHandle = GDRV_init(&GdrvObj,sizeof(GdrvObj));\n") 
        if Registry.PowerstageProtectionConfig["gpio"] == nil then
            PreInitCode:append("  DIO_Handle_t dioHandle = DIO_obtainDummyWrite();\n")
            PreInitCode:append("  GDRV_assignPin(GdrvHandle, GDRV_Pin_EnableOut, dioHandle, GDRV_Pin_ActiveLow);\n") 
        else
            PreInitCode:append("  DIO_Handle_t dioHandle = DIO_init(&DoutDrvEnableObj,sizeof(DoutDrvEnableObj));\n") 
            if Registry.PowerstageProtectionConfig["pol"] == false then
                PreInitCode:append("  DIO_configureOut(dioHandle, %i, true);\n" % {Registry.PowerstageProtectionConfig["gpio"]})
                PreInitCode:append("  GDRV_assignPin(GdrvHandle, GDRV_Pin_EnableOut, dioHandle, GDRV_Pin_ActiveLow);\n")
            else
                PreInitCode:append("  DIO_configureOut(dioHandle, %i, false);\n" % {Registry.PowerstageProtectionConfig["gpio"]})
                PreInitCode:append("  GDRV_assignPin(GdrvHandle, GDRV_Pin_EnableOut, dioHandle, GDRV_Pin_ActiveHigh);\n")
            end
        end
        PreInitCode:append("  GDRV_powerup(GdrvHandle);\n") 
        PreInitCode:append("  PWR_configure(GdrvHandle, %i);\n" % {ps_rate}) 
        PreInitCode:append("}\n") 
    end

    -- configure PWMs
    error = C.ConfigurePwms(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end

    -- configure ADCs
    error = C.ConfigureAdcs(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end

    -- configure QEPs
    error = C.ConfigureQeps(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end

    -- configure SPIs
    error = C.ConfigureSpis(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end
    
    -- configure PGAs
    error = T.ConfigurePgas(PreInitCode)
    if error ~= nil then
        return error
    end

    -- configure DACs
    error = C.ConfigureDacs(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end

    -- configure CAPs
    error = C.ConfigureCaps(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end
	
	-- configure CAN ports
	error = C.ConfigureCan(Registry, PreInitCode, T)
    if error ~= nil then
        return error
    end

    -- process templates
    local dict = {}
    table.insert(dict, {before = "|>BASE_NAME<|", after = Target.Variables.BASE_NAME})
    table.insert(dict, {before = "|>TARGET_ROOT<|", after = Target.Variables.TARGET_ROOT})
    U.CopyTemplateFile(Target.Variables.TARGET_ROOT .. "/templates/settings.h", "%s/%s_settings_cg.h" % {installDir, Target.Variables.BASE_NAME}, dict)

    if Target.Variables.genOnly == 2 then
    --[[
        "One-click" build and flash
    --]]
        local codegenDir = Target.FamilySettings.ExternalTools.codegenDir:gsub('%"+', ''):gsub('\\+','/') -- remove quotes, make all forward slashes
        if not U.FileExists(codegenDir) then
            return "Codegen tools directory '%s' not found." % {codegenDir}
        end
        
        local uniflashFile
        local board = T.GetBoardNameFromComboIndex(Target.Variables.board)
        if board ~= 'custom' then
            uniflashFile = Target.Variables.TARGET_ROOT .. "/templates/uniflash/" .. T.GetUniflashConfig(board)
        else
            uniflashFile = Target.Variables.uniflashFile
        end

        local uniflashDir = Target.FamilySettings.ExternalTools.uniflashDir:gsub('%"+', ''):gsub('\\+','/') -- remove quotes, make all forward slashes
        if not U.FileExists(uniflashDir) then
            return "Uniflash directory '%s' not found." % {uniflashDir}
        end
        
        if not U.FileExists(uniflashFile) then
            return "Uniflash configuration file '%s' not found." % {uniflashFile}
        end
        
        table.insert(dict, {before = "|>INSTALL_DIR<|", after = './'})
        table.insert(dict, {before = "|>BIN_DIR<|", after = "./output_%s" % {Target.Name}})
        table.insert(dict, {before = "|>CG_PATH<|", after = codegenDir})
        local srcDir = Target.Variables.TARGET_ROOT .. "/" .. Target.Name
        table.insert(dict, {before = "|>SRC_ROOT<|", after = srcDir})
        
        local uniflashExe = ""
        if Target.Variables.HOST_OS == "win" then
            uniflashExe = "%s/dslite.bat" % {uniflashDir}
        else
            uniflashExe = "%s/dslite.sh" % {uniflashDir}
        end
        if not U.FileExists(uniflashExe) then
            return "Uniflash executable '%s' not found." % {uniflashExe}
        end

        if uniflashFile:sub(-#".ccxml") == ".ccxml" then
            table.insert(dict, {before = "|>CCXML_FILE<|", after = uniflashFile})
        else
            table.insert(dict, {before = "|>CCXML_FILE<|", after = ""})
        end

        table.insert(dict, {before = "|>FLASH_EXE<|", after = uniflashExe})


        -- handle instaSPIN as a special case
		local mk_postfix = ''
        if Registry.GenericInstances['EST'] ~= nil then
			if Target.Variables.buildConfig ~= 1 then
				return "'Run from RAM' not supported for models with InstaSpin block."
			end
			mk_postfix = '_instaspin'
			table.insert(dict, {before = "|>INSTASPIN<|", after = "YES"})
		else
			table.insert(dict, {before = "|>INSTASPIN<|", after = "NO"})
        end
        
        -- handle Uniflash version number   

        local uniflashVerPath = Target.FamilySettings.ExternalTools.uniflashDir .. "/uniflash/public/version.txt" 
        local uniflashVersion
        if not U.FileExists(uniflashVerPath) then
           uniflashVersion = 6.0  --recommended UniFlash version in the C2000 manual 
        else
           local file = io.open(uniflashVerPath, "rb"):read "*all"  -- r read mode and b binary mode ; *all reads the whole file   
              if file == '' then
                uniflashVersion = 6.0  --recommended UniFlash version in the C2000 manual 
              else 
                uniflashVersion = tonumber(file:sub(1, 3));     
              end
        end
       
        if Target.Variables.buildConfig == 1 then
            table.insert(dict, {before = "|>BUILD_CONFIG<|", after = "FLASH"})
            U.CopyTemplateFile(srcDir .. "/templates/flash%s.lkf" % {mk_postfix}, Target.Variables.BUILD_ROOT .. "/" .. Target.Variables.BASE_NAME .. ".lkf", dict)
        else
            table.insert(dict, {before = "|>BUILD_CONFIG<|", after = "RAM"})
            U.CopyTemplateFile(srcDir .. "/templates/ram%s.lkf" % {mk_postfix}, Target.Variables.BUILD_ROOT .. "/" .. Target.Variables.BASE_NAME .. ".lkf", dict)
        end
        
         if uniflashVersion < 6.1 then  --https://e2e.ti.com/support/microcontrollers/c2000/f/171/p/933199/3448467#3448467 
            table.insert(dict, {before = "|>AUTO_START_OPTION<|", after = ""})
            U.CopyTemplateFile(srcDir .. "/templates/main.mk", Target.Variables.BUILD_ROOT .. "/" .. Target.Variables.BASE_NAME .. ".mk", dict)
        else
            table.insert(dict, {before = "|>AUTO_START_OPTION<|", after = "-u"})
            U.CopyTemplateFile(srcDir .. "/templates/main.mk", Target.Variables.BUILD_ROOT .. "/" .. Target.Variables.BASE_NAME .. ".mk", dict)
        end
        
    else
    --[[
        Generate files into CCS and build/debug from there
    --]]
        table.insert(dict, {before = "|>INSTALL_DIR<|", after = installDir})
        table.insert(dict, {before = "|>RUN_PIL_PREP<|", after = 0}) -- PIL Prep Tool will likely not be needed
        local pilPrepDir = ''
        U.CopyTemplateFile(Target.Variables.TARGET_ROOT .. "/templates/install.mk", Target.Variables.BUILD_ROOT .. "/" .. Target.Variables.BASE_NAME .. ".mk", dict)
        if Target.Variables.HOST_OS == "win" then
            table.insert(dict, {before = "|>BUILD_ROOT<|", after = Target.Variables.BUILD_ROOT:gsub('/+','\\')}) -- in batch file, we want backslashes
            table.insert(dict, {before = "|>PIL_PREP_DIR<|", after = pilPrepDir:gsub('/+','\\')})
            U.CopyTemplateFile(Target.Variables.TARGET_ROOT .. "/templates/buildsteps.bat", installDir .. "/buildsteps.bat", dict)
        else
            table.insert(dict, {before = "|>BUILD_ROOT<|", after = Target.Variables.BUILD_ROOT})
            table.insert(dict, {before = "|>PIL_PREP_DIR<|", after = pilPrepDir})
            U.CopyTemplateFile(Target.Variables.TARGET_ROOT .. "/templates/buildsteps.sh", installDir.. "/buildsteps.sh", dict)
        end

        -- helper file to specify model base-name to CCS
        U.CopyTemplateFile(Target.Variables.TARGET_ROOT .. "/templates/basename_cg.h", installDir .. "/basename_cg.h", dict)
    end

    local file = io.open(Target.Variables.BUILD_ROOT .. "/settings.tmp", "w")
    io.output(file)
    
    -- version ID 
    local mav, miv = string.match(Target.Version, '(%d+).(%d+)')
    if mav ~= nil and miv ~=nil then
        local versionHex = 256 * tonumber(mav) + tonumber(miv)
        io.write("// TSP Version\n")
        io.write("#define TSP_VER 0x%X\n\n" % {versionHex})
    end

    -- clock settings
    error = T.WriteClockConfiguration(io)
    if error ~= nil then
        return error
    end
    
    -- background task
    if #Registry.BackgroundCodeDeclarations ~= 0 then
        Declarations:append("/* background task declarations */\n")
        for _, line in ipairs(Registry.BackgroundCodeDeclarations) do
            Declarations:append(line)
        end
        Declarations:append("\n")
    end
    Declarations:append("void %s_background(void)\n" % {Target.Variables.BASE_NAME})
    Declarations:append("{\n")
    if Registry.BackgroundTaskCount ~= 0 then
        Declarations:append("static int task = 0;\n")
        Declarations:append("switch(task){\n")
        for task, code in pairs(Registry.BackgroundCode) do
            Declarations:append("    case %i: /* %s */\n" % {task, Registry.BackgroundTaskLabels[task]})
            Declarations:append("    {\n")
            for _, line in ipairs(code) do
                Declarations:append("  %s" % {line})
            end
            Declarations:append("    }\n")
            Declarations:append("    break;\n")
        end
        Declarations:append("    default:\n")
        Declarations:append("        break;\n")
        Declarations:append("}\n")
        Declarations:append("task++;\n")
        Declarations:append("if(task >= %i){\n" % {Registry.BackgroundTaskCount})
        Declarations:append("    task = 0;\n")
        Declarations:append("}\n")
    end
    Declarations:append("\n}\n")

    -- memory allocation
    io.write("// HAL object allocations\n")
    io.write("#define HAL_NUM_DOUT_CHANNELS " .. Registry.NumDigitalOutBlocks .. "\n")
    io.write("#define HAL_NUM_DIN_CHANNELS " .. Registry.NumDigitalInBlocks .. "\n")
    io.write("#define HAL_NUM_PWM_CHANNELS " .. Registry.NumPwmBlocks .. "\n")
    io.write("#define HAL_NUM_ADCS " .. Registry.NumAdcBlocks .. "\n")
    io.write("#define HAL_NUM_QEP_CHANNELS " .. Registry.NumQepBlocks .. "\n")
    io.write("#define HAL_NUM_TZ_CHANNELS 6\n") -- FIXME: No magic numbers!
    io.write("#define HAL_NUM_SPI_CHANNELS " .. Registry.NumSpiBlocks .. "\n") 
    io.write("#define HAL_NUM_DAC_CHANNELS " .. Registry.NumDacBlocks .. "\n") 
    io.write("#define HAL_NUM_CAP_CHANNELS " .. Registry.NumCapBlocks .. "\n") 
	io.write("#define HAL_NUM_CANBUS_CHANNELS " .. Registry.NumCanHandles .. "\n")
	
    for id, count in pairs(Registry.GenericInstances) do
      io.write("#define HAL_NUM_%s_INSTANCES %i \n" % {id, count}) 
    end
    io.write("\n")

    -- dispatcher (basetask) settings
    local achievableModelPeriodInSysClk, achievableModelClkHz
    error, achievableModelPeriodInSysClk, achievableModelClkHz = C.WriteDispatcherConfiguration(io, Registry, T)
    if error ~= nil then
        return error
    end

    -- register tasks
    local taskFunction
    if #Model.Tasks == 1 then
        taskFunction = [[
        static void Tasks(void * const aParam)
        {
            %s_step();
        }
        ]]
    elseif #Model.Tasks > 16 then
        return "Maximal allowable number of tasks (16) exceeded."
    else
        taskFunction = [[
        static void Tasks(void * const aParam)
        {
            %s_step(*(int *)aParam);
        }
        ]]
    end
    Declarations:append("%s\n" % {taskFunction % {Target.Variables.BASE_NAME}})    
    local numTasks = 0
    for idx = 1, #Model.Tasks do
        local tsk = Model.Tasks[idx]
        local ts = tsk["SampleTime"]
        if ts[2] ~= 0 then
            return "Sample time offset not supported."
        end
        
        local achievablePeriodInSysClk = achievableModelPeriodInSysClk * math.floor(achievableModelClkHz*ts[1] + 0.5)
        PreInitCode:append("{\n")
        PreInitCode:append("    static int taskId = %i;\n" % {numTasks})
        PreInitCode:append("    // Task %i at %e Hz\n" % {numTasks, (Target.Variables.sysClkMHz * 1e6)/achievablePeriodInSysClk});
        PreInitCode:append("    DISPR_registerTask(%i, &Tasks, %iL, (void *)&taskId);\n" % {numTasks, achievablePeriodInSysClk});
        PreInitCode:append("}\n")
        numTasks = numTasks + 1
    end
    io.write("#define NUM_TASKS %d\n" % {numTasks})
    io.write("\n")

    -- external mode comm interface
    if(Target.Variables.EXTERNAL_MODE ~= 0) then
        io.write("// External mode settings\n")
        io.write("#define EXT_MODE_BUFFER_SIZE %d\n" % {Target.Variables.extModeBufferSize})
        io.write("#define NUM_EXT_MODE_SIGNALS %d\n" % {Model.NumExtModeSignals})
    end
    
    if(Target.Variables.EXTERNAL_MODE == 1) then
        -- "Serial"
        if(#Target.Variables.extModeSciPins ~= 2) then
            return 'Exactly two SCI pins must be specified.'
        end

        -- GPIO used for serial communication
        local sciPinset = T.GetSciPinset("%d-%d" % {Target.Variables.extModeSciPins[1], Target.Variables.extModeSciPins[2]})
        if sciPinset == nil then
            return 'SCI pin-set not supported.'
        end
        Require:add("GPIO", Target.Variables.extModeSciPins[1], "External mode communication (Rx)")
        Require:add("GPIO", Target.Variables.extModeSciPins[2], "External mode communication (Tx)")

        -- determine baud rate
        local sciBaud
        local maxRate = T.GetMaxSciBaudRate()
        local supportedRates = {256000, 128000, 115200, 57600, 38400, 19200, 14400, 9600, 4800}
        for _, rate in ipairs(supportedRates) do
            if maxRate >= rate then
                sciBaud = rate
                break
            end
        end

        if sciBaud == nil then
             return "The control task execution rate is too low to support external mode communication."
        end
        
        -- save settings
        io.write("#define SCI_BAUD_RATE %d\n" % {sciBaud})
        io.write("#define SCI_PORT %d\n" % {sciPinset})
    elseif (Target.Variables.EXTERNAL_MODE == 2) then
        -- "JTAG"
        io.write("#define PARALLEL_COM_PROTOCOL %d\n" % {3}) -- 3 = non-intrusive
    end
    io.write("\n")

    io.close(file)
    U.CopyTemplateFile(Target.Variables.BUILD_ROOT .. "/settings.tmp", "%s/%s_settings.inc" % {installDir, Target.Variables.BASE_NAME})
    os.remove(Target.Variables.BUILD_ROOT .. "/settings.tmp")

    return {
        Declarations = Declarations,
        PreInitCode = PreInitCode,
        PostInitCode = PostInitCode,
        TerminateCode = TerminateCode,
        Include = Include,
        Require = Require
    }
end

return Coder
