local T = { }

-- target options
local boards = {'custom', 'launchpad', 'controlcard'} -- must match info.xml file
local uniFlashConfigs = {
    launchpad = 'Launchpad_TMS320F280049C.ccxml',
    controlcard = 'ControlCard_TMS320F280049C.ccxml'
}

function T.GetBoardNameFromComboIndex(index)
    return boards[index]
end

function T.GetUniflashConfig(board)
    return uniFlashConfigs[board]
end

-- resources
local pwmGpio = {}
pwmGpio[1] = {0,1}
pwmGpio[2] = {2,3}
pwmGpio[3] = {4,5}
pwmGpio[4] = {6,7}
pwmGpio[5] = {8,9}
pwmGpio[6] = {10,11}    
pwmGpio[7] = {12,13}
pwmGpio[8] = {14,15}  

local qepPinset = {}
qepPinset["28-29-59"] = 10
qepPinset["35-37-59"] = 11

qepPinset["14-15-26"] = 20

local spiPinset = {}
spiPinset["A-16-17-56-57"] = 10
spiPinset["B-24-31-22-33"] = 20
spiPinset["A-16-17-56"] = 10
spiPinset["B-24-31-22"] = 20

local sciPinset = {}
sciPinset["28-29"] = 0
sciPinset["35-37"] = 1

local canPinset = {}
canPinset["A-33-32"] = 0

function T.Configure(resources, registry)
  resources:add("Model Trigger", 1, 1)  
  resources:add("Powerstage Control", 1, 1)  	
  resources:add("GPIO", 0, 59)    
  resources:add("ADC", 0, 2)   
  resources:add("PWM", 1, 8) 
  resources:add("QEP", 1, 2)  
  resources:add("SPI-A")
  resources:add("SPI-B")
  resources:add("ADCA-SOC", 0, 15)
  resources:add("ADCB-SOC", 0, 15)  
  resources:add("ADCC-SOC", 0, 15)
  resources:add("DAC", 0, 1)
  resources:add("EST", 0, 0)
  resources:add("CAP", 1, 7)
end

function T.GetPwmGpio(unit)
	return pwmGpio[unit]
end

function T.CheckGpioIsValidPwmSync(gpio)
	if gpio > 59 then
		return "Only GPIO from 0 to 59 can be selected as external synchronisation source for 28004x."
	end
end

function T.GetSciPinset(pinsetS)
	return sciPinset[pinsetS]
end

function T.GetQepPinset(pinsetS)
	return qepPinset[pinsetS]
end

function T.GetSpiPinset(pinsetS)
	return spiPinset[pinsetS]
end

function T.CheckSpiClockIsAchievable(clk)
	local lspClkHz = Target.Variables.sysClkMHz * 1e6 / 4 -- LSPCLK_HZ hardcoded below
	local maxClk = math.floor(lspClkHz/(0x03+1))
	local minClk = math.ceil(lspClkHz/(0x7F+1))
	
	if clk > maxClk then
		return "SPI clock rate must not exceed %d Hz." % {maxClk}
	elseif clk < minClk then
		return "SPI clock rate must not be below %d Hz." % {minClk}
	end
end

function T.CheckGpioIsValidCapPin(cap, gpio, req)
    -- all pins can be used (XBAR)
end

function T.GetCanPinset(pinsetS)
	return canPinset[pinsetS]
end

function T.CheckCanConfigIsValid(canInterface)
	if canInterface > 1 then
		return "CAN %s not supported." % {string.char(65+canInterface)}
	end
end

function T.GetIntOscClock()
	return T.GetTargetParameters()['intosc_hz']
end

function T.GetTimerClock()
    -- to help with margin and latency measurements, timer is configured at rate of PWM
	return T.GetPwmClock()
end

function T.GetPwmClock()
	-- TBCLOCK - CLKDIV=/1, HSPCLKDIV=/2
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	return sysClkHz / 2
end

function T.GetDeadTimeClock()
	-- TBCLOCK - CLKDIV=/1, HSPCLKDIV=/2 
	-- TBCLK = EPWMCLK/(HSPCLKDIV * CLKDIV)
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	return sysClkHz / 2 
end

function T.GetMaxEventPeriod()
	return T.GetTargetParameters()['max_event_period']
end

function T.GetAdcClock()
	-- HAL automatically configures ADCCTL2.bit.PRESCALE
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	if sysClkHz <= 50000000 then
		return sysClkHz
	else
		return sysClkHz / 2
	end
end

function T.GetMaxSciBaudRate()
	-- assuming 8N1. 1.5 characters per poll
	local maxRate = 1/Target.Variables.SAMPLE_TIME * 15
	return math.min(115200, maxRate)
end

function T.CalcACQPS(ts)
	local tsMin = math.max(75e-9, 1/T.GetAdcClock())-- per datasheet
	if ts ~= ts or ts < tsMin then -- test for NAN or value that is too small
		ts = tsMin 
	end
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	return math.floor(ts * sysClkHz)
end

function T.GetINTSEL1N2RegisterName()
	return T.GetTargetParameters()['intsel1n2_reg_name']
end

function T.GetCanClkAndMaxBrp()
	return Target.Variables.sysClkMHz*1e6, 0x400
end

function T.ConfigurePgas(PreInitCode)
  local gains = {3, 6, 12, 24};
  local rfs = {0, 200, 160, 130, 100, 80, 50}
  local pdaConfs = {}
  for i=1,7 do
    if Target.Variables["pga%iEn" % {i}] == 1 then
      local gainCombo = Target.Variables["pga%iGain" % {i}]
      local rfCombo = Target.Variables["pga%iRf" % {i}]
      pdaConfs[i] = {gain = gains[gainCombo], rf = rfs[rfCombo]}
    end
  end
  
  if(#pdaConfs ~= 0) then
    PreInitCode:append("// configurable gain amplifiers (PGAs)\n")
    for i in pairs(pdaConfs) do
      PreInitCode:append("HAL_enablePga(%i, %f, %f);\n" % {i, pdaConfs[i].gain, pdaConfs[i].rf})
    end
  end
end

function T.GetDacOutputRange()
	local range = T.GetTargetParameters()['dac_out_range'] 
	return range[1], range[2] 
end

function T.GetTargetParameters()
	params = {
		spi_fifo_depth = 16,
		epwm_type = 4,
		intosc_hz = 10000000,
		intsel1n2_reg_name = 'ADCINTSEL1N2',
		max_event_period = 15,
		dac_out_range = {0.0, 3.3}, --Min/Max output voltage for this target
		num_extsync_in = 2
	}
	return params
end

function T.WriteClockConfiguration(io)
  local sysClkHz = Target.Variables.sysClkMHz * 1e6
	
  local clkin, clksrc
  
  if sysClkHz > 100000000 then
    return "Excessive system clock setting."
  elseif sysClkHz < 2000000 then
    return "Insufficient system clock setting."
  end 
  
  if sysClkHz ~= math.floor(sysClkHz) then
    return "System clock setting must be integer value."
  end 

  if Target.Variables.useIntOsc ~= 1 then
  	clkin = Target.Variables.extClkMHz * 1e6
  	clksrc = 1
  else
  	clkin =  T.GetIntOscClock()
  	clksrc = 0
  end
  
  local odivRegVal = 0 -- hard-coded
  local odiv = odivRegVal + 1
  
  local sysClkDivRegVal = 1 -- hard-coded  
  local sysClkDiv = math.max(1, sysClkDivRegVal*2)
 
  local fmult = 0 -- TODO: allow different values for more flexibility below
  local fmultRegVal = fmult/0.25
  
  local imult = math.min(math.floor(odiv*sysClkDiv*sysClkHz/clkin - fmult), 127)
  local imultRegVal = imult
  
  local vcoMHz = (imult + fmult) * clkin / 1000000
  if (vcoMHz < 120) or (vcoMHz > 400) then
     return "PLL VCO frequency (%d MHz) outside of valid range (120-400 MHz)." % {vcoMHz}
  end
  
  local pllRawMHz = vcoMHz / odiv
  if (pllRawMHz < 15) or (pllRawMHz > 200) then
     return "PLL raw frequency (%d MHz) outside of valid range (15-200 Mhz)." % {pllRawMHz}
  end
  
  if not (sysClkHz == (imult + fmult)/odiv * clkin/sysClkDiv) then
    return "Unable to achieve the desired system clock frequency (with input clock = %d Hz)." % {clkin}
  end
  
  local cpuRate = 100.0/ sysClkHz * 10000000

  io.write("// Clock configurations\n")
  io.write("#define SYSCLK_HZ %iL\n" % {sysClkHz}) 
  io.write("#define LSPCLK_HZ (SYSCLK_HZ / 4l)\n")
    
  io.write("#define PLL_SYSCLK_DIV %i\n" % {sysClkDivRegVal}) 
  io.write("#define PLL_FMULT %i\n" % {fmultRegVal})
  io.write("#define PLL_IMULT %i\n" % {imultRegVal})
  io.write("#define PLL_ODIV %i\n" % {odivRegVal})  
  io.write("#define PLL_SRC " .. clksrc ..  "\n")  
  io.write("#define PLX_CPU_RATE " .. cpuRate .. "L\n")
  io.write("\n")
end

return T
