local T = { }

-- target options
local boards = {'custom'} -- must match info.xml file
local uniFlashConfigs = {
}

function T.GetBoardNameFromComboIndex(index)
    return boards[index]
end

function T.GetUniflashConfig(board)
    return uniFlashConfigs[board]
end

-- resources
local pwmGpio = {}
pwmGpio[1] = {0,1}
pwmGpio[2] = {2,3}
pwmGpio[3] = {4,5}
pwmGpio[4] = {6,7}
pwmGpio[5] = {8,9}
pwmGpio[6] = {10,11}  

local qepPinset = {}
qepPinset["20-21-23"] = 0
qepPinset["50-51-53"] = 1

local spiPinset = {}
spiPinset["A-16-17-18-19"] = 10
spiPinset["A-16-17-18"] = 10

local sciPinset = {}
sciPinset["28-29"] = 0

local capPins = {}
capPins[1] = 6
capPins[3] = 5
capPins[5] = 1
capPins[7] = 2
capPins[9] = 3
capPins[11] = 4
capPins[24] = 1
capPins[25] = 2
capPins[26] = 3
capPins[27] = 4
capPins[34] = 1
capPins[37] = 2
capPins[48] = 5
capPins[49] = 6

local canPinset = {}
canPinset["A-30-31"] = 0
canPinset["B-21-20"] = 10

function T.Configure(resources, registry)
  resources:add("Model Trigger", 1, 1)  
  resources:add("Powerstage Control", 1, 1)  	
  resources:add("GPIO", 0, 86) 
  resources:add("ADC", 0, 1)   
  resources:add("PWM", 1, 6) 
  resources:add("QEP", 1, 2) 
  resources:add("SPI-A")
  resources:add("SPI-B")
  resources:add("ADCA-SOC", 0, 15)
  resources:add("CAP", 1, 6)
end

function T.GetPwmGpio(unit)
	return pwmGpio[unit]
end

function T.CheckGpioIsValidPwmSync(gpio)
	if gpio~=6 and gpio~=32 then
		return "Only GPIO 6 and 32 can be selected as external synchronisation source for 2833x."
	end
end

function T.GetSciPinset(pinsetS)
	return sciPinset[pinsetS]
end

function T.GetQepPinset(pinsetS)
	return qepPinset[pinsetS]
end

function T.GetSpiPinset(pinsetS)
	return spiPinset[pinsetS]
end

function T.CheckSpiClockIsAchievable(clk)
	local lspClkHz = Target.Variables.sysClkMHz * 1e6 / 6 -- LSPCLK_HZ hardcoded below
	local maxClk = math.floor(lspClkHz/(0x03+1))
	local minClk = math.ceil(lspClkHz/(0x7F+1))
	
	if clk > maxClk then
		return "SPI clock rate must not exceed %d Hz." % {maxClk}
	elseif clk < minClk then
		return "SPI clock rate must not be below %d Hz." % {minClk}
	end
end

function T.CheckGpioIsValidCapPin(cap, gpio, req)
    if capPins[gpio] ~= cap then
        return "GPIO %i is not a valid pin for CAP %i." % {gpio, cap}
    end
    req:add("GPIO", gpio)  
end

function T.GetCanPinset(pinsetS)
	return canPinset[pinsetS]
end

function T.CheckCanConfigIsValid(canInterface)
	if canInterface > 1 then
		return "CAN %s not supported." % {string.char(65+canInterface)}
	end
end

function T.GetIntOscClock()
	return 10000000 -- INTOSC2
end

function T.GetTimerClock()
    -- to help with margin and latency measurements, timer is configured at rate of PWM
	return T.GetPwmClock()
end

function T.GetPwmClock()
	return Target.Variables.sysClkMHz * 1e6 / 2
end

function T.GetDeadTimeClock()
	return Target.Variables.sysClkMHz * 1e6 / 2
end

function T.GetMaxEventPeriod()
	return 3
end

function T.GetMaxSciBaudRate()
	-- assuming 8N1. 1.5 characters per poll
	local maxRate = 1/Target.Variables.SAMPLE_TIME * 15
	return math.min(115200, maxRate)
end

function T.GetAdcClock()
	-- HAL automatically configures ADCCTL2.bit.PRESCALE
	local sysClkHz = Target.Variables.sysClkMHz * 1e6
	if sysClkHz <= 45000000 then
		return sysClkHz
	else
		return sysClkHz / 2
	end
end

function T.CalcACQPS(ts)
	local tsMin = 7/T.GetAdcClock() -- per datasheet - different
	local tsMax = 64/T.GetAdcClock()
	if ts ~= ts or ts < tsMin then -- test for NAN or value that is too small
		ts = tsMin 
	elseif ts > tsMax then
		ts = tsMax -- maybe we should be less tolerant here and throw an error?
	end
	return math.floor(ts * T.GetAdcClock());
end

function T.GetINTSEL1N2RegisterName()
	return "INTSEL1N2"
end

function T.GetCanClkAndMaxBrp()
	return Target.Variables.sysClkMHz*1e6/2, 0x40
end

function T.ConfigurePgas(PreInitCode)
  -- this chip does not have PGAs
end

function T.GetDacOutputRange()
  --This chip does not have DAC
end

function T.GetTargetParameters()
	params = {
		spi_fifo_depth = 16,
		epwm_type = 0,
		intosc_hz = nil,
		intsel1n2_reg_name = 'INTSEL1N2',
		max_event_period = 3,
		num_extsync_in = 1
	}
	return params
end

function T.WriteClockConfiguration(io)
  local sysClkHz = Target.Variables.sysClkMHz * 1e6
  
  if sysClkHz > 150000000 then
    return "Excessive system clock setting."
  end 
  
  if sysClkHz ~= math.floor(sysClkHz) then
    return "System clock setting must be integer value."
  end 

  local clkin, clksrc

  if Target.Variables.useIntOsc ~= 1 then
  	clkin = Target.Variables.extClkMHz * 1e6
  	clksrc = 1
  else
  	clkin =  T.GetIntOscClock()
  	clksrc = 0
  end


  local pllDiv = math.min(math.floor(2*sysClkHz/clkin), 127)
  if not (sysClkHz == pllDiv * clkin/2) then
    return "Unable to achieve the desired system clock frequency (with input clock = %d Hz)." % {clkin}
  end
  
  local cpuRate = 100.0/ sysClkHz * 10000000;
  
  io.write("// Clock configurations\n")
  io.write("#define SYSCLK_HZ %iL\n" % {sysClkHz}) 
  io.write("#define LSPCLK_HZ (SYSCLK_HZ / 6l)\n")
    
  io.write("#define PLL_DIV %i\n" % {pllDiv})
  io.write("#define PLX_CPU_RATE " .. cpuRate .. "L\n")
  io.write("\n")
end

return T
